@@ -27,7 +27,6 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <sys/cdefs.h>
 
 /*
  * svc_vc.c, Server side for Connection Oriented based RPC. 
@@ -41,7 +40,7 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/param.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <sys/un.h>
 #include <sys/time.h>
 #include <sys/uio.h>
@@ -310,7 +309,6 @@ rendezvous_request(xprt, msg)
 	socklen_t len;
 	struct __rpc_sockinfo si;
 	SVCXPRT *newxprt;
-	fd_set cleanfds;
 
 	assert(xprt != NULL);
 	assert(msg != NULL);
@@ -322,13 +320,16 @@ again:
 	    &len)) < 0) {
 		if (errno == EINTR)
 			goto again;
-		/*
-		 * Clean out the most idle file descriptor when we're
-		 * running out.
-		 */
+
 		if (errno == EMFILE || errno == ENFILE) {
-			cleanfds = svc_fdset;
-			__svc_clean_idle(&cleanfds, 0, FALSE);
+		  /* If there are no file descriptors available, then accept will fail.
+		     We want to delay here so the connection request can be dequeued;
+		     otherwise we can bounce between polling and accepting, never
+		     giving the request a chance to dequeue and eating an enormous
+		     amount of cpu time in svc_run if we're polling on many file
+		     descriptors.  */
+		        struct timespec ts = { .tv_sec = 0, .tv_nsec = 50000000 };
+                        nanosleep (&ts, NULL);
 			goto again;
 		}
 		return (FALSE);
@@ -338,6 +339,8 @@ again:
 	 */
 
 	newxprt = makefd_xprt(sock, r->sendsize, r->recvsize);
+	if (!newxprt)
+		return (FALSE);
 
 	if (!__rpc_set_netbuf(&newxprt->xp_rtaddr, &addr, len))
 		return (FALSE);
@@ -393,6 +396,12 @@ svc_vc_destroy(xprt)
 	__svc_vc_dodestroy(xprt);
 }
 
+static bool_t
+__svc_rendezvous_socket(xprt)
+	SVCXPRT *xprt;
+{
+	return (xprt->xp_ops->xp_recv == rendezvous_request);
+}
 static void
 __svc_vc_dodestroy(xprt)
 	SVCXPRT *xprt;
@@ -404,7 +413,7 @@ __svc_vc_dodestroy(xprt)
 
 	if (xprt->xp_fd != RPC_ANYFD)
 		(void)close(xprt->xp_fd);
-	if (xprt->xp_port != 0) {
+	if (__svc_rendezvous_socket(xprt)) {
 		/* a rendezvouser socket */
 		r = (struct cf_rendezvous *)xprt->xp_p1;
 		mem_free(r, sizeof (struct cf_rendezvous));
@@ -787,47 +796,17 @@ __rpc_get_local_uid(SVCXPRT *transp, uid_t *uid) {
  * rpcbind are known to call this function.  Do not alter or remove this
  * API without changing the library's sonum.
  */
+/* Since this is an exported interface used by rpcbind, we cannot
+   remove it. But since poll() can handle much more and much higher
+   file descriptors, this code doesn't really work anymore, too.
+   So for now, keep it as dummy function and do nothing to not break
+   existing binaries. If we have ported rpcbind to the poll() interface
+   and find out, that we really need this cleanup stuff (but nobody
+   besides FreeBSD has this), we need to re-implement it using poll().
+   But this means a new function name with different parameters. For
+   ABI/API compatibility, we cannot reuse this one. */
 bool_t
 __svc_clean_idle(fd_set *fds, int timeout, bool_t cleanblock)
 {
-	int i, ncleaned;
-	SVCXPRT *xprt, *least_active;
-	struct timeval tv, tdiff, tmax;
-	struct cf_conn *cd;
-
-	gettimeofday(&tv, NULL);
-	tmax.tv_sec = tmax.tv_usec = 0;
-	least_active = NULL;
-	rwlock_wrlock(&svc_fd_lock);
-	for (i = ncleaned = 0; i <= svc_maxfd; i++) {
-		if (FD_ISSET(i, fds)) {
-			xprt = __svc_xports[i];
-			if (xprt == NULL || xprt->xp_ops == NULL ||
-			    xprt->xp_ops->xp_recv != svc_vc_recv)
-				continue;
-			cd = (struct cf_conn *)xprt->xp_p1;
-			if (!cleanblock && !cd->nonblock)
-				continue;
-			if (timeout == 0) {
-				timersub(&tv, &cd->last_recv_time, &tdiff);
-				if (timercmp(&tdiff, &tmax, >)) {
-					tmax = tdiff;
-					least_active = xprt;
-				}
-				continue;
-			}
-			if (tv.tv_sec - cd->last_recv_time.tv_sec > timeout) {
-				__xprt_unregister_unlocked(xprt);
-				__svc_vc_dodestroy(xprt);
-				ncleaned++;
-			}
-		}
-	}
-	if (timeout == 0 && least_active != NULL) {
-		__xprt_unregister_unlocked(least_active);
-		__svc_vc_dodestroy(least_active);
-		ncleaned++;
-	}
-	rwlock_unlock(&svc_fd_lock);
-	return ncleaned > 0 ? TRUE : FALSE;
+	return FALSE;
 }
